@@ -825,20 +825,6 @@ ipa_bts_id_resp(const struct ipaccess_unit *dev, uint8_t *data, int len, int trx
 	return nmsg;
 }
 
-static struct msgb *ipa_bts_id_ack(void)
-{
-	struct msgb *nmsg2;
-
-	nmsg2 = ipa_msg_alloc(0);
-	if (!nmsg2)
-		return NULL;
-
-	*msgb_put(nmsg2, 1) = IPAC_MSGT_ID_ACK;
-	ipa_msg_push_header(nmsg2, IPAC_PROTO_IPACCESS);
-
-	return nmsg2;
-}
-
 static void ipaccess_bts_updown_cb(struct ipa_client_conn *link, int up)
 {
 	struct e1inp_line *line = link->line;
@@ -864,64 +850,60 @@ int ipaccess_bts_handle_ccm(struct ipa_client_conn *link,
 			    struct ipaccess_unit *dev, struct msgb *msg)
 {
 	struct ipaccess_head *hh = (struct ipaccess_head *) msg->data;
-	struct msgb *rmsg;
+
+	/* special handling for IPA CCM. */
+	if (hh->proto != IPAC_PROTO_IPACCESS)
+		return 0;
+
 	int ret = 0;
+	uint8_t *data = msgb_l2(msg);
+	int len = msgb_l2len(msg);
+	OSMO_ASSERT(len > 0);
+	uint8_t msg_type = *data;
 	/* line might not exist if != bsc||bts */
 	struct e1inp_line *line = link->line;
 
-	/* special handling for IPA CCM. */
-	if (hh->proto == IPAC_PROTO_IPACCESS) {
-		uint8_t msg_type = *(msg->l2h);
-		struct osmo_fsm_inst* ka_fsm = NULL;
-
-		/* peek the pong for our keepalive fsm */
-		if (line && msg_type == IPAC_MSGT_PONG) {
-			ka_fsm = ipaccess_line_ts(link->ofd, line)->driver.ipaccess.ka_fsm;
-			ipa_keepalive_fsm_pong_received(ka_fsm);
-		}
-
-		/* ping, pong and acknowledgment cases. */
-		ret = ipa_ccm_rcvmsg_bts_base(msg, link->ofd);
-		if (ret < 0)
-			goto err;
-
-		/* this is a request for identification from the BSC. */
-		if (msg_type == IPAC_MSGT_ID_GET) {
-			uint8_t *data = msgb_l2(msg);
-			int len = msgb_l2len(msg);
-			int trx_nr = 0;
-
-			if (link->ofd->priv_nr >= E1INP_SIGN_RSL)
-				trx_nr = link->ofd->priv_nr - E1INP_SIGN_RSL;
-
-			LOGP(DLINP, LOGL_NOTICE, "received ID_GET for unit ID %u/%u/%u\n",
-			     dev->site_id, dev->bts_id, trx_nr);
-			rmsg = ipa_bts_id_resp(dev, data + 1, len - 1, trx_nr);
-			ret = ipa_send(link->ofd->fd, rmsg->data, rmsg->len);
-			if (ret != rmsg->len) {
-				LOGP(DLINP, LOGL_ERROR, "cannot send ID_RESP message. Reason: %s\n",
-				     strerror(errno));
-				goto err_rmsg;
-			}
-			msgb_free(rmsg);
-
-			/* send ID_ACK. */
-			rmsg = ipa_bts_id_ack();
-			ret = ipa_send(link->ofd->fd, rmsg->data, rmsg->len);
-			if (ret != rmsg->len) {
-				LOGP(DLINP, LOGL_ERROR, "cannot send ID_ACK message. Reason: %s\n",
-				     strerror(errno));
-				goto err_rmsg;
-			}
-			msgb_free(rmsg);
-		}
-		return 1;
+	/* peek the pong for our keepalive fsm */
+	if (line && msg_type == IPAC_MSGT_PONG) {
+		struct osmo_fsm_inst *ka_fsm = ipaccess_line_ts(link->ofd, line)->driver.ipaccess.ka_fsm;
+		ipa_keepalive_fsm_pong_received(ka_fsm);
 	}
 
-	return 0;
+	/* ping, pong and acknowledgment cases. */
+	ret = ipa_ccm_rcvmsg_bts_base(msg, link->ofd);
+	if (ret < 0)
+		goto err;
+
+	/* this is a request for identification from the BSC. */
+	if (msg_type == IPAC_MSGT_ID_GET) {
+		struct msgb *rmsg;
+		int trx_nr = 0;
+
+		if (link->ofd->priv_nr >= E1INP_SIGN_RSL)
+			trx_nr = link->ofd->priv_nr - E1INP_SIGN_RSL;
+
+		LOGP(DLINP, LOGL_NOTICE, "received ID_GET for unit ID %u/%u/%u\n",
+			dev->site_id, dev->bts_id, trx_nr);
+		rmsg = ipa_bts_id_resp(dev, data + 1, len - 1, trx_nr);
+		ret = ipa_send(link->ofd->fd, rmsg->data, rmsg->len);
+		if (ret != rmsg->len) {
+			LOGP(DLINP, LOGL_ERROR, "cannot send ID_RESP message. Reason: %s\n",
+				strerror(errno));
+			msgb_free(rmsg);
+			goto err;
+		}
+		msgb_free(rmsg);
+
+		/* send ID_ACK. */
+		ret = ipa_ccm_send_id_ack(link->ofd->fd);
+		if (ret <= 0) {
+			LOGP(DLINP, LOGL_ERROR, "cannot send ID_ACK message. Reason: %s\n",
+				strerror(errno));
+			goto err;
+		}
+	}
+	return 1;
 
-err_rmsg:
-	msgb_free(rmsg);
 err:
 	ipa_client_conn_close(link);
 	return -1;
@@ -955,9 +937,9 @@ static int ipaccess_bts_read_cb(struct ipa_client_conn *link, struct msgb *msg)
 
 	if (ret == 1 && hh->proto == IPAC_PROTO_IPACCESS) {
 		if (msg_type == IPAC_MSGT_ID_GET) {
-			sign_link = link->line->ops->sign_link_up(msg,
-					link->line,
-					link->ofd->priv_nr);
+			sign_link = link->line->ops->sign_link_up(link->line->ops->cfg.ipa.dev,
+								  link->line,
+								  link->ofd->priv_nr);
 			if (sign_link == NULL) {
 				LOGP(DLINP, LOGL_ERROR,
 					"Unable to set signal link, "
@@ -967,11 +949,12 @@ static int ipaccess_bts_read_cb(struct ipa_client_conn *link, struct msgb *msg)
 		}
 		msgb_free(msg);
 		return ret;
-	} else if (link->port == IPA_TCP_PORT_OML)
+	}
+
+	if (link->port == IPA_TCP_PORT_OML)
 		e1i_ts = e1inp_line_ipa_oml_ts(link->line);
 	else if (link->port == IPA_TCP_PORT_RSL)
 		e1i_ts = e1inp_line_ipa_rsl_ts(link->line, link->ofd->priv_nr - E1INP_SIGN_RSL);
-
 	OSMO_ASSERT(e1i_ts != NULL);
 
 	if (e1i_ts->type == E1INP_TS_TYPE_NONE) {
